/* -*- compile-command: "msdev modeler.dsp /make \"Modeler - Win32 Debug\"" -*-
 */

#include <math.h>

#include "modeler.h"

#define BOX_DISPLAY_LIST       1

/* sphere
 *
 * draw sphere with given radius.
 */

void sphere( double r )
{
    GLUquadric* gluq;

    /* let the GLU library do all the work for us.  the g_quality
       variable determines how many polygons the sphere is sliced up
       into. */
    
    gluq = gluNewQuadric();
    gluQuadricDrawStyle( gluq, GLU_FILL );
    gluSphere( gluq, r, g_quality, g_quality );
    gluDeleteQuadric( gluq );
}

/* box
 *
 * draws a box with opposite corners at the origin and at (x,y,z).
 */

void box( double x, double y, double z )
{
    static int displaylist = -1;

    /* remember which matrix mode OpenGL was in. */
    int savemode;
    glGetIntegerv( GL_MATRIX_MODE, &savemode );

    /* switch to the model matrix and scale by x,y,z. */
    glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glScaled( x, y, z );

    /* now we just need to draw a cube from 0,0,0 to 1,1,1.  we'll
       save it in a display list the first time we draw it, to make
       subsequent boxes easy and fast. */
    
    if ( displaylist == -1 )
    {
	/* this is the first time we've drawn a box, so save the
	   commands we execute in a display list. */
	
	glNewList( BOX_DISPLAY_LIST, GL_COMPILE_AND_EXECUTE );

	/* draw each face of the box as a quad.  this is pretty
	   straightforward. */

	glBegin( GL_QUADS );

	glNormal3d( 0.0, 0.0, -1.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 1.0, 0.0 );
	glVertex3d( 1.0, 1.0, 0.0 );
	glVertex3d( 1.0, 0.0, 0.0 );
	
	glNormal3d( 0.0, -1.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 1.0, 0.0, 0.0 );
	glVertex3d( 1.0, 0.0, 1.0 );
	glVertex3d( 0.0, 0.0, 1.0 );
	
	glNormal3d( -1.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 1.0 );
	glVertex3d( 0.0, 1.0, 1.0 );
	glVertex3d( 0.0, 1.0, 0.0 );
	
	glNormal3d( 0.0, 0.0, 1.0 );
	glVertex3d( 0.0, 0.0, 1.0 );
	glVertex3d( 1.0, 0.0, 1.0 );
	glVertex3d( 1.0, 1.0, 1.0 );
	glVertex3d( 0.0, 1.0, 1.0 );
	
	glNormal3d( 0.0, 1.0, 0.0 );
	glVertex3d( 0.0, 1.0, 0.0 );
	glVertex3d( 0.0, 1.0, 1.0 );
	glVertex3d( 1.0, 1.0, 1.0 );
	glVertex3d( 1.0, 1.0, 0.0 );
	
	glNormal3d( 1.0, 0.0, 0.0 );
	glVertex3d( 1.0, 0.0, 0.0 );
	glVertex3d( 1.0, 1.0, 0.0 );
	glVertex3d( 1.0, 1.0, 1.0 );
	glVertex3d( 1.0, 0.0, 1.0 );

	glEnd();

	/* all done!  stop saving commands to the display list. */
	glEndList();
    }
    else
    {
	/* this is not the first time we've drawn a box, so all we
	   need to do is evaulate the display list. */
	
	glCallList( BOX_DISPLAY_LIST );
    }

    /* restore the model matrix stack, and switch back to the matrix
       mode we were in. */
    glPopMatrix();
    glMatrixMode( savemode );
}

/* cylinder
 *
 * cylinder draws a round frustum along the z-axis, with radius r1 at
 * the z=0 end and radius r2 at the z=h end.  a disk cap is drawn at any
 * end with a nonzero radius.
 */

void cylinder( double h, double r1, double r2 )
{
    GLUquadric* gluq;

    /* GLU will again do the work.  draw the sides of the cylinder. */
    
    gluq = gluNewQuadric();
    gluQuadricDrawStyle( gluq, GLU_FILL );
    gluCylinder( gluq, r1, r2, h, g_quality, g_quality/2 );
    gluDeleteQuadric( gluq );

    if ( r1 > 0.0 )
    {
	/* if the r1 end does not come to a point, draw a flat disk to
	   cover it up. */
	
	gluq = gluNewQuadric();
	gluQuadricDrawStyle( gluq, GLU_FILL );
	gluQuadricOrientation( gluq, GLU_INSIDE );
	gluDisk( gluq, 0.0, r1, g_quality, g_quality/2 );
	gluDeleteQuadric( gluq );
    }
    
    if ( r2 > 0.0 )
    {
	/* if the r2 end does not come to a point, draw a flat disk to
	   cover it up. */

	/* save the current matrix mode. */	
	int savemode;
	glGetIntegerv( GL_MATRIX_MODE, &savemode );

	/* translate the origin to the other end of the cylinder. */
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glTranslated( 0.0, 0.0, h );

	/* draw a disk centered at the new origin. */
	gluq = gluNewQuadric();
	gluQuadricDrawStyle( gluq, GLU_FILL );
	gluQuadricOrientation( gluq, GLU_OUTSIDE );
	gluDisk( gluq, 0.0, r2, g_quality, g_quality/2 );
	gluDeleteQuadric( gluq );

	/* restore the matrix stack and mode. */
	glPopMatrix();
	glMatrixMode( savemode );
    }
}
    
/* triangle
 *
 * draws a flat triangle, given three vertices.
 */

void triangle( double x1, double y1, double z1,
	       double x2, double y2, double z2,
	       double x3, double y3, double z3 )
{
    double a, b, c, d, e, f;

    /* the normal to the triangle is the cross product
       of two of its edges. */
    
    a = x2-x1;
    b = y2-y1;
    c = z2-z1;

    d = x3-x1;
    e = y3-y1;
    f = z3-z1;

    glBegin( GL_TRIANGLES );
    glNormal3d( b*f - c*e, c*d - a*f, a*e - b*d );
    glVertex3d( x1, y1, z1 );
    glVertex3d( x2, y2, z2 );
    glVertex3d( x3, y3, z3 );
    glEnd();
}

	       

